<?php

abstract class File
{
    public $path;
    public $filename;
    /**
     * @param array|string $data
     * @return Image
     * @throws ErrorException
     */
    public static function getInstance($data)
    {
        if (is_null($data) || $data == '') {
            throw new ErrorException('Empty data for File::getInstance().');
        }
        if (!is_array($data)) {
            $data = json_decode($data, true);
            if (json_last_error() != JSON_ERROR_NONE) {
                throw new ErrorException('Unable to convert json-string to array.');
            }
        }
        $instance = new static($data);
        foreach ($data as $attribute_name => $attribute_value) {
            if (property_exists($instance, $attribute_name)) {
                $instance->{$attribute_name} = $attribute_value;
            }
        }
        return $instance;
    }

    /**
     * @param $size string
     * @return ImageVariant
     */
    public function getImageVariant($size)
    {
        if (!array_key_exists($size, $this->variants)) {
            $this->variants[$size] = new ImageVariant;
        } else {
            if (!is_object($this->variants[$size])) {
                $this->variants[$size] = self::getInstance($this->variants[$size]);
            }
        }
        return $this->variants[$size];
    }

    /**
     * @return string
     */
    public function getWebName()
    {
        return $this->path . '/' . $this->filename;
    }

    public function __toString()
    {
        return FileHelper::toString($this);
    }
}